﻿// JScript File

if(typeof RINDEX == 'undefined' || typeof RINDEX.Widget == 'undefined')
	throw new Error('Required file "RIndex.js" was missing.');

ClientTable = function(tableToConvert, excludeRowClassNameArr, hiddenColumnIndexArr) {
  
  this.table = RINDEX.Utility.getObject(tableToConvert);
	
	this.selectedRows = [];
	
	this.disabled = false;
	
	/**
	 * @field multiSelection
	 * 获取一个值，该值指示是否可以同时选中多行
	 *@type Boolean
	 */
	this.multiSelection = true;
	
	this.excludeClassName = excludeRowClassNameArr;
	this.hiddenColumnIndex = hiddenColumnIndexArr; // 要隐藏的列

	if(this.table != null) {
	  this.table.onselectstart = function() {return false;};
	  RINDEX.Utility.formatTable(this.table);
	  RINDEX.Utility.addCSS(this.table,'editTable');
	  this._init();
	}
  
};


ClientTable.prototype = {

  // Events
  onRowSelected : function(row) {
		return true;
	},
	
  onSelectChange : function(row) {
		return true;
	},
	
	onDblClick : function(row) {
		return true;
	},

  _init : function() {
    this._format();
    if(!this.disabled) {
      this._attachEvent();
    }
  },
  
  _format : function() {
    if(this.table.rules)
      this.table.rules = 'none';
    var rowCount = this.table.rows.length;
    var cellCount = null;
    var tblHeader, tblRow;
    
    // Hide the haeder row
    tblHeader = this.table.rows[0];
    if(0 in tblHeader.cells) {
      if(tblHeader.cells[0].tagName.toUpperCase() == 'TH') {
        this._hidColume(this.table.rows[0]);
      }
    }
    
    // Hide all other rows
    for(var i=1; i<rowCount; i++) {
      tblRow = this.table.rows[i];
      this._hidColume(tblRow);
      
      if(this._skipRow(tblRow))
        continue;
      if(cellCount == null) {
        cellCount = tblRow.cells.length;
      }
      for(var cell=0; cell<cellCount; cell++){
        tblRow.cells[cell].className = 'item-' + ((i+1)%2);
      }
    }
  },
  
  _hidColume : function(rowEL) {
    if(this.hiddenColumnIndex && this.hiddenColumnIndex.length) {
      var indexCount = this.hiddenColumnIndex.length;
      if(indexCount>0) {
      
        if(this._skipRow(rowEL)) {
          rowEL.childNodes[0].colSpan -= indexCount;
        } else {
          for(var cell=0; cell<rowEL.cells.length; cell++) {
            for(var index=0; index<indexCount; index++){
              if(this.hiddenColumnIndex[index] == cell) {
                rowEL.cells[cell].style.display = 'none';
              }
            }
          }
        }
        
      }
    }
  },
  
  _attachEvent : function(event) {
    var rowCount = this.table.rows.length;
    var tblRow; var self = this;
    event = event || window.event;
    for(var i=1; i<rowCount; i++) {
      tblRow = this.table.rows[i];
      if(this._skipRow(tblRow))
        continue;
      tblRow.onclick = function(event) {
        self.select(this, event);
      };
      tblRow.ondblclick = function() {
        self.onDblClick(this);
      };
    } 
  },
  
  _skipRow : function(rowEL) {
    if(rowEL.className && this.excludeClassName && this.excludeClassName.length) {
      for(var i=0; i<this.excludeClassName.length; i++) {
        if(this.excludeClassName == rowEL.className)
          return true;
      }
    }
    return false;
  },
  
  select : function(row, event) {
	
		event = event || window.event;
			
		if(RINDEX.Lang.isNumber(row))
			row = this.table.rows[row];
			
		if(this.multiSelection == false) {
		
		  if(this.selectedRows.length > 0) {
		    if(this.selectedRows[0] != row) {
		      this.unSelect(this.selectedRows[0]);
		      this.selectedRows = [row];
		      this._appendSelectStyle(row);
		      this.onRowSelected(row);
					this.onSelectChange(row);
		    }
		  } else {
		    this.selectedRows = [row];
		    this._appendSelectStyle(row);
		    this.onRowSelected(row);
				this.onSelectChange(row);
		  }
		}
		
		else {
			
			var contains = false;
			for(var i=0; i<this.selectedRows.length; i++) {
				if(this.selectedRows[i] == row) {
					contains = true;
					break;
				}
			}

			if(event.ctrlKey) {
				
				if(!contains) {
					this.selectedRows.push(row);
					this._appendSelectStyle(row);
					this.onRowSelected(row);
					this.onSelectChange(row);
				} else {
					this.unSelect(row);
				}
			}
			
			else if(event.shiftKey) {
			
			  var baseRow = 0;
			  if(this.selectedRows.length > 0)
			    baseRow = this.selectedRows[this.selectedRows.length-1].rowIndex
			    
			  var nwArr = [];
			  var rowIndex = row.rowIndex;
			  
			  if(baseRow < rowIndex) {
					for(var value = rowIndex; value>=baseRow; value--) {
							nwArr.push(value);
					}
				} else {
					for(var value = rowIndex; value<=baseRow; value++) {
							nwArr.push(value);
					}
				}

				this._clearAllSelection();
				
				for(var i=0; i<nwArr.length; i++) {
				  this.selectedRows.push(this.table.rows[nwArr[i]]);
				  this._appendSelectStyle(this.table.rows[nwArr[i]]);
				}
				
				this.onSelectChange(row);
				
			}
			
			else {
				
				this._clearAllSelection();
				
				this.selectedRows.push(row);
				this._appendSelectStyle(row);
				this.onRowSelected(row);
				this.onSelectChange(row);
			}
			
		}
		
		RINDEX.Utility.refreshControl();
		
	},
	
	unSelect : function(row) {
	  if(this.disabled) return row;
		if(RINDEX.Lang.isNumber(row))
			row = this.table.rows[row];
			
		var index = -1;
		for(var i=0; i<this.selectedRows.length; i++) {
			if(this.selectedRows[i] == row) {
				index = i;
				break;
			}
		}
		
		if(index > -1) {
			this.selectedRows =
				this.selectedRows.slice(0,index).concat(this.selectedRows.slice(index+1));
			this._clearSelectStyle(row);
			this.onSelectChange(row);
		}
			
	},
	
	moveUp : function(rowEL) { 
	  var currentRowIndex = rowEL.rowIndex;
	  if(currentRowIndex<2)
	    return;
	  var nwRow = this.table.insertRow(currentRowIndex-1);
	  var tempCell, rawCell;
	  for(var i=0; i<rowEL.cells.length; i++) {
	    rawCell = rowEL.cells[i];
	    tempCell = nwRow.insertCell(i);
	    tempCell.innerHTML = rawCell.innerHTML;
	    tempCell.className = rawCell.className;
	  }
	  this.remove(rowEL);
	  this._attachEvent();
	  this.select(nwRow);
	},
	
	moveDown : function(rowEL) {
	  var currentRowIndex = rowEL.rowIndex;
	  if(currentRowIndex==this.table.rows.length-1)
	    return;
	  var nwRow = this.table.insertRow(currentRowIndex+2);
	  var tempCell, rawCell;
	  for(var i=0; i<rowEL.cells.length; i++) {
	    rawCell = rowEL.cells[i];
	    tempCell = nwRow.insertCell(i);
	    tempCell.innerHTML = rawCell.innerHTML;
	    tempCell.className = rawCell.className;
	  }
	  this.remove(rowEL);
	  this._attachEvent();
	  this.select(nwRow);
	},
	
	remove : function(row) {
	  if(this.disabled) return row;
		if(RINDEX.Lang.isNumber(row))
			row = this.table.rows[row];
		try {
			this.unSelect(row);
			this.table.deleteRow(row.rowIndex);
			this._format();
		} catch(e) {
			//
		}
	},
	
	removeSelectedRows : function() {
	  if(this.disabled) return;
		while(this.selectedRows.length > 0)
			this.remove(this.selectedRows[0]);
	},
	
	getCellText : function(cellIndex) {
	  var rowCount = this.selectedRows.length;
	  var arr = new Array(rowCount);
	  for(var i=0; i<rowCount; i++) {
	    if(document.all)
	      arr[i] = this.selectedRows[i].cells[cellIndex].innerText;
	    else
	      arr[i] = this.selectedRows[i].cells[cellIndex].textContent;
	  }
	  return arr;
	},
	
	changeHeaderText : function(textArr) {
	  var headerRow = this.table.rows[0];
	  var cellCount = headerRow.cells.length-1;
	  var tempCellEL;
	  for(var i=0; i<textArr.length; i++) {
	    if(i > cellCount)
	      continue;
	    tempCellEL = headerRow.cells[i];
	    if(tempCellEL.tagName.toUpperCase() == 'TH') {
	      tempCellEL.innerHTML = textArr[i];
	    }
	  }
	},
	
	_appendSelectStyle : function(row) {
		
		for(var cell=0; cell<row.cells.length; cell++) {
			RINDEX.Utility.addCSS(row.cells[cell], 'selected');
		}
		
	},
	
	_clearSelectStyle : function(row) {
		
		for(var cell=0; cell<row.cells.length; cell++) {
			RINDEX.Utility.removeCSS(row.cells[cell], 'selected');
		}
		
	},
	
	_clearAllSelection : function() {
		for(var i=0; i<this.selectedRows.length; i++) {
			this._clearSelectStyle(this.selectedRows[i]);
		}
		this.selectedRows.length = 0;
	},

  toString : function() {
    return '[ClientTable]';
  }

};